@@ -15,7 +15,11 @@
 module DataFrame.Functions where
 
 import DataFrame.Internal.Column
-import DataFrame.Internal.DataFrame (DataFrame (..), unsafeGetColumn)
+import DataFrame.Internal.DataFrame (
+    DataFrame (..),
+    columnAsDoubleVector,
+    unsafeGetColumn,
+ )
 import DataFrame.Internal.Expression (
     Expr (..),
     UExpr (..),
@@ -30,6 +34,7 @@ import DataFrame.Operations.Subset (exclude, select)
 import Control.Exception (throw)
 import Control.Monad
 import qualified Data.Char as Char
+import Data.Containers.ListUtils
 import Data.Function
 import qualified Data.List as L
 import qualified Data.Map as M
@@ -37,8 +42,10 @@ import Data.Maybe (fromMaybe, listToMaybe)
 import qualified Data.Set as S
 import qualified Data.Text as T
 import Data.Type.Equality
+import qualified Data.Vector as V
 import qualified Data.Vector.Generic as VG
 import qualified Data.Vector.Unboxed as VU
+import qualified DataFrame.Operations.Core as D
 import qualified DataFrame.Operations.Transformations as D
 import Debug.Trace (trace, traceShow)
 import Language.Haskell.TH
@@ -81,47 +88,67 @@ div = BinaryOp "div" Prelude.div
 mod :: (Integral a, Columnable a) => Expr a -> Expr a -> Expr a
 mod = BinaryOp "mod" Prelude.mod
 
-(==) :: (Columnable a, Eq a) => Expr a -> Expr a -> Expr Bool
-(==) = BinaryOp "eq" (Prelude.==)
+(.==) :: (Columnable a, Eq a) => Expr a -> Expr a -> Expr Bool
+(.==) = BinaryOp "eq" (==)
 
 eq :: (Columnable a, Eq a) => Expr a -> Expr a -> Expr Bool
-eq = BinaryOp "eq" (Prelude.==)
+eq = BinaryOp "eq" (==)
 
-(<) :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
-(<) = BinaryOp "lt" (Prelude.<)
+(.<) :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
+(.<) = BinaryOp "lt" (<)
 
 lt :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
-lt = BinaryOp "lt" (Prelude.<)
+lt = BinaryOp "lt" (<)
 
-(>) :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
-(>) = BinaryOp "gt" (Prelude.>)
+(.>) :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
+(.>) = BinaryOp "gt" (>)
 
 gt :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
-gt = BinaryOp "gt" (Prelude.>)
+gt = BinaryOp "gt" (>)
 
-(<=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
-(<=) = BinaryOp "leq" (Prelude.<=)
+(.<=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
+(.<=) = BinaryOp "leq" (<=)
 
 leq :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
-leq = BinaryOp "leq" (Prelude.<=)
+leq = BinaryOp "leq" (<=)
 
-(>=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
-(>=) = BinaryOp "geq" (Prelude.>=)
+(.>=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
+(.>=) = BinaryOp "geq" (>=)
 
 geq :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
-geq = BinaryOp "geq" (Prelude.>=)
+geq = BinaryOp "geq" (>=)
 
 and :: Expr Bool -> Expr Bool -> Expr Bool
 and = BinaryOp "and" (&&)
 
+(.&&) :: Expr Bool -> Expr Bool -> Expr Bool
+(.&&) = BinaryOp "and" (&&)
+
 or :: Expr Bool -> Expr Bool -> Expr Bool
 or = BinaryOp "or" (||)
 
+(.||) :: Expr Bool -> Expr Bool -> Expr Bool
+(.||) = BinaryOp "or" (||)
+
 not :: Expr Bool -> Expr Bool
 not = UnaryOp "not" Prelude.not
 
 count :: (Columnable a) => Expr a -> Expr Int
-count expr = AggFold expr "foldUdf" 0 (\acc _ -> acc + 1)
+count expr = AggFold expr "count" 0 (\acc _ -> acc + 1)
+
+collect :: (Columnable a) => Expr a -> Expr [a]
+collect expr = AggFold expr "collect" [] (flip (:))
+
+mode :: (Columnable a, Eq a) => Expr a -> Expr a
+mode expr =
+    AggVector
+        expr
+        "mode"
+        ( fst
+            . L.maximumBy (compare `on` snd)
+            . M.toList
+            . V.foldl' (\m e -> M.insertWith (+) e 1 m) M.empty
+        )
 
 minimum :: (Columnable a, Ord a) => Expr a -> Expr a
 minimum expr = AggReduce expr "minimum" Prelude.min
@@ -146,7 +173,7 @@ percentile n expr =
     AggNumericVector
         expr
         (T.pack $ "percentile " ++ show n)
-        ((VU.! 0) . quantiles' (VU.fromList [n]) 100)
+        (percentile' n)
 
 stddev :: (Columnable a, Real a, VU.Unbox a) => Expr a -> Expr Double
 stddev expr = AggNumericVector expr "stddev" (sqrt . variance')
@@ -155,7 +182,9 @@ zScore :: Expr Double -> Expr Double
 zScore c = (c - mean c) / stddev c
 
 pow :: (Columnable a, Num a) => Int -> Expr a -> Expr a
-pow i = UnaryOp ("pow " <> T.pack (show i)) (^ i)
+pow 0 _ = Lit 1
+pow 1 expr = expr
+pow i expr = UnaryOp ("pow " <> T.pack (show i)) (^ i) expr
 
 relu :: (Columnable a, Num a) => Expr a -> Expr a
 relu = UnaryOp "relu" (Prelude.max 0)
@@ -171,76 +200,108 @@ reduce ::
     (Columnable a, Columnable b) => Expr b -> a -> (a -> b -> a) -> Expr a
 reduce expr = AggFold expr "foldUdf"
 
-generatePrograms :: [Expr Double] -> [Expr Double] -> [Expr Double]
-generatePrograms vars [] =
-    vars
-        ++ [ transform p
-           | p <- vars
-           , transform <-
-                [ zScore
-                , abs
-                , sqrt
-                , log . (+ Lit 1)
-                , exp
-                , mean
-                , median
-                , stddev
-                , sin
-                , cos
-                , relu
-                , signum
-                ]
-           ]
-        ++ [ pow i p
-           | p <- vars
-           , i <- [2 .. 6]
-           ]
-        ++ [ p + q
-           | (i, p) <- zip [0 ..] vars
-           , (j, q) <- zip [0 ..] vars
-           , i Prelude.> j
-           ]
-        ++ [ DataFrame.Functions.min p q
-           | (i, p) <- zip [0 ..] vars
-           , (j, q) <- zip [0 ..] vars
-           , i Prelude.> j
-           ]
-        ++ [ DataFrame.Functions.max p q
-           | (i, p) <- zip [0 ..] vars
-           , (j, q) <- zip [0 ..] vars
-           , i Prelude.>= j
-           ]
-        ++ [ p - q
-           | (i, p) <- zip [0 ..] vars
-           , (j, q) <- zip [0 ..] vars
-           , i /= j
-           ]
-        ++ [ p * q
-           | (i, p) <- zip [0 ..] vars
-           , (j, q) <- zip [0 ..] vars
-           , i Prelude.>= j
-           ]
-        ++ [ p / q
-           | (i, p) <- zip [0 ..] vars
-           , (j, q) <- zip [0 ..] vars
-           , i /= j
-           ]
-generatePrograms vars ps =
+generateConditions ::
+    TypedColumn Double -> [Expr Bool] -> [Expr Double] -> DataFrame -> [Expr Bool]
+generateConditions labels conds ps df =
     let
-        existingPrograms = vars ++ ps
+        newConds =
+            [ p .<= q
+            | p <- ps
+            , q <- ps
+            , p /= q
+            ]
+                ++ [ DataFrame.Functions.not p
+                   | p <- conds
+                   ]
+        expandedConds =
+            conds
+                ++ newConds
+                ++ [p .&& q | p <- newConds, q <- conds, p /= q]
+                ++ [p .|| q | p <- newConds, q <- conds, p /= q]
+     in
+        pickTopNBool df labels (deduplicate df expandedConds)
+
+generatePrograms ::
+    [Expr Bool] -> [Expr Double] -> [Expr Double] -> [Expr Double] -> [Expr Double]
+generatePrograms conds vars' constants [] =
+    let
+        vars = vars' ++ constants
+     in
+        nubOrd $
+            vars
+                ++ [ transform p
+                   | p <- vars'
+                   , transform <-
+                        [ abs
+                        , sqrt
+                        , log . (+ Lit 1)
+                        , exp
+                        , sin
+                        , cos
+                        , relu
+                        , signum
+                        ]
+                   ]
+                ++ [ pow i p
+                   | p <- vars
+                   , i <- [2 .. 6]
+                   ]
+                ++ [ p + q
+                   | (i, p) <- zip [0 ..] vars
+                   , (j, q) <- zip [0 ..] vars
+                   , Prelude.not (isLiteral p && isLiteral q)
+                   , i > j
+                   ]
+                ++ [ DataFrame.Functions.min p q
+                   | (i, p) <- zip [0 ..] vars
+                   , (j, q) <- zip [0 ..] vars
+                   , p /= q
+                   , Prelude.not (isLiteral p && isLiteral q)
+                   , i > j
+                   ]
+                ++ [ DataFrame.Functions.max p q
+                   | (i, p) <- zip [0 ..] vars
+                   , (j, q) <- zip [0 ..] vars
+                   , p /= q
+                   , Prelude.not (isLiteral p && isLiteral q)
+                   , i > j
+                   ]
+                ++ [ ifThenElse cond r s
+                   | cond <- conds
+                   , r <- vars
+                   , s <- vars
+                   , r /= s
+                   ]
+                ++ [ p - q
+                   | (i, p) <- zip [0 ..] vars
+                   , (j, q) <- zip [0 ..] vars
+                   , Prelude.not (isLiteral p && isLiteral q)
+                   , i /= j
+                   ]
+                ++ [ p * q
+                   | (i, p) <- zip [0 ..] vars
+                   , (j, q) <- zip [0 ..] vars
+                   , Prelude.not (isLiteral p && isLiteral q)
+                   , i >= j
+                   ]
+                ++ [ p / q
+                   | (i, p) <- zip [0 ..] vars
+                   , (j, q) <- zip [0 ..] vars
+                   , Prelude.not (isLiteral p && isLiteral q)
+                   , i /= j
+                   ]
+generatePrograms conds vars constants ps =
+    let
+        existingPrograms = ps ++ vars ++ constants
      in
         existingPrograms
             ++ [ transform p
-               | p <- existingPrograms
+               | p <- ps ++ vars
                , transform <-
-                    [ zScore
-                    , sqrt
+                    [ sqrt
                     , abs
                     , log . (+ Lit 1)
                     , exp
-                    , mean
-                    , median
-                    , stddev
                     , sin
                     , cos
                     , relu
@@ -254,51 +315,59 @@ generatePrograms vars ps =
             ++ [ p + q
                | (i, p) <- zip [0 ..] existingPrograms
                , (j, q) <- zip [0 ..] existingPrograms
-               , i Prelude.>= j
+               , Prelude.not (isLiteral p && isLiteral q)
+               , i >= j
                ]
             ++ [ DataFrame.Functions.min p q
                | (i, p) <- zip [0 ..] existingPrograms
                , (j, q) <- zip [0 ..] existingPrograms
-               , i Prelude.> j
+               , Prelude.not (isLiteral p && isLiteral q)
+               , p /= q
+               , i > j
                ]
             ++ [ DataFrame.Functions.max p q
                | (i, p) <- zip [0 ..] existingPrograms
                , (j, q) <- zip [0 ..] existingPrograms
-               , i Prelude.> j
+               , Prelude.not (isLiteral p && isLiteral q)
+               , p /= q
+               , i > j
                ]
-            ++ [ ifThenElse (p DataFrame.Functions.>= percentile n p) p q
-               | (i, p) <- zip [0 ..] existingPrograms
-               , (j, q) <- zip [0 ..] existingPrograms
-               , i /= j
-               , n <- [1, 25, 50, 75, 99]
-               ]
-            ++ [ ifThenElse (p DataFrame.Functions.>= percentile n p) p q
-               | p <- existingPrograms
-               , q <- [lit 1, lit 0, lit (-1)]
-               , n <- [1, 25, 50, 75, 99]
+            ++ [ ifThenElse cond r s
+               | cond <- conds
+               , r <- existingPrograms
+               , s <- existingPrograms
+               , r /= s
                ]
             ++ [ p - q
                | (i, p) <- zip [0 ..] existingPrograms
                , (j, q) <- zip [0 ..] existingPrograms
+               , Prelude.not (isLiteral p && isLiteral q)
                , i /= j
                ]
             ++ [ p * q
                | (i, p) <- zip [0 ..] existingPrograms
                , (j, q) <- zip [0 ..] existingPrograms
-               , i Prelude.>= j
+               , Prelude.not (isLiteral p && isLiteral q)
+               , i >= j
                ]
             ++ [ p / q
-               | (i, p) <- zip [0 ..] existingPrograms
-               , (j, q) <- zip [0 ..] existingPrograms
-               , i /= j
+               | p <- existingPrograms
+               , q <- existingPrograms
+               , Prelude.not (isLiteral p && isLiteral q)
+               , p /= q
                ]
 
--- | Deduplicate programs pick the least smallest one by size.
+isLiteral :: Expr a -> Bool
+isLiteral (Lit _) = True
+isLiteral _ = False
+
 deduplicate ::
+    forall a.
+    (Columnable a) =>
     DataFrame ->
-    [Expr Double] ->
-    [(Expr Double, TypedColumn Double)]
-deduplicate df = go S.empty . L.sortBy (\e1 e2 -> compare (eSize e1) (eSize e2))
+    [Expr a] ->
+    [(Expr a, TypedColumn a)]
+deduplicate df = go S.empty . nubOrd . L.sortBy (\e1 e2 -> compare (eSize e1) (eSize e2))
   where
     go _ [] = []
     go seen (x : xs)
@@ -306,18 +375,18 @@ deduplicate df = go S.empty . L.sortBy (\e1 e2 -> compare (eSize e1) (eSize e2))
         | S.member res seen = go seen xs
         | otherwise = (x, res) : go (S.insert res seen) xs
       where
-        res = case interpret df x of
+        res = case interpret @a df x of
             Left e -> throw e
             Right v -> v
         hasInvalid = case res of
-            (TColumn (UnboxedColumn (col :: VU.Vector a))) -> case testEquality (typeRep @Double) (typeRep @a) of
+            (TColumn (UnboxedColumn (col :: VU.Vector b))) -> case testEquality (typeRep @Double) (typeRep @b) of
                 Just Refl -> VU.any (\n -> isNaN n || isInfinite n) col
                 Nothing -> False
             _ -> False
 
 -- | Checks if two programs generate the same outputs given all the same inputs.
 equivalent :: DataFrame -> Expr Double -> Expr Double -> Bool
-equivalent df p1 p2 = case (Prelude.==) <$> interpret df p1 <*> interpret df p2 of
+equivalent df p1 p2 = case (==) <$> interpret df p1 <*> interpret df p2 of
     Left e -> throw e
     Right v -> v
 
@@ -338,6 +407,8 @@ synthesizeFeatureExpr target cfg df =
             df'
             cfg
             t
+            (percentiles df')
+            []
             [] of
             Nothing -> Left "No programs found"
             Just p -> Right p
@@ -346,7 +417,7 @@ f1FromBinary :: VU.Vector Double -> VU.Vector Double -> Maybe Double
 f1FromBinary trues preds =
     let (!tp, !fp, !fn) =
             VU.foldl' step (0 :: Int, 0 :: Int, 0 :: Int) $
-                VU.zip (VU.map (Prelude.> 0) preds) (VU.map (Prelude.> 0) trues)
+                VU.zip (VU.map (> 0) preds) (VU.map (> 0) trues)
      in f1FromCounts tp fp fn
   where
     step (!tp, !fp, !fn) (!p, !t) =
@@ -361,9 +432,9 @@ f1FromCounts tp fp fn =
     let tp' = fromIntegral tp
         fp' = fromIntegral fp
         fn' = fromIntegral fn
-        precision = if tp' + fp' Prelude.== 0 then 0 else tp' / (tp' + fp')
-        recall = if tp' + fn' Prelude.== 0 then 0 else tp' / (tp' + fn')
-     in if precision + recall Prelude.== 0
+        precision = if tp' + fp' == 0 then 0 else tp' / (tp' + fp')
+        recall = if tp' + fn' == 0 then 0 else tp' / (tp' + fn')
+     in if precision + recall == 0
             then Nothing
             else Just (2 * precision * recall / (precision + recall))
 
@@ -387,9 +458,20 @@ fitClassifier target d b df =
             df'
             (BeamConfig d b F1)
             t
+            (percentiles df' ++ [lit 1, lit 0, lit (-1)])
+            []
             [] of
             Nothing -> Left "No programs found"
-            Just p -> Right (ifThenElse (p DataFrame.Functions.> 0) 1 0)
+            Just p -> Right (ifThenElse (p .> 0) 1 0)
+
+percentiles :: DataFrame -> [Expr Double]
+percentiles df =
+    let
+        doubleColumns = map (either throw id . (`columnAsDoubleVector` df)) (D.columnNames df)
+     in
+        concatMap (\c -> map (lit . (`percentile'` c)) [1, 23, 75, 99]) doubleColumns
+            ++ map (lit . variance') doubleColumns
+            ++ map (lit . sqrt . variance') doubleColumns
 
 fitRegression ::
     -- | Target expression
@@ -416,6 +498,8 @@ fitRegression target d b df =
                 MutualInformation
             )
             t
+            (percentiles df')
+            []
             [] of
             Nothing -> Left "No programs found"
             Just p ->
@@ -427,7 +511,9 @@ fitRegression target d b df =
                             )
                             (BeamConfig d b MeanSquaredError)
                             t
-                            [Col "_generated_regression_feature_", lit targetMean, lit 10] of
+                            (percentiles df' ++ [lit targetMean, lit 10])
+                            []
+                            [Col "_generated_regression_feature_"] of
                             Nothing -> Left "Could not find coefficients"
                             Just p' -> Right (replaceExpr p (Col @Double "_generated_regression_feature_") p')
 
@@ -466,11 +552,15 @@ beamSearch ::
     BeamConfig ->
     -- | Examples
     TypedColumn Double ->
+    -- | Constants
+    [Expr Double] ->
+    -- | Conditions
+    [Expr Bool] ->
     -- | Programs
     [Expr Double] ->
     Maybe (Expr Double)
-beamSearch df cfg outputs programs
-    | searchDepth cfg Prelude.== 0 = case ps of
+beamSearch df cfg outputs constants conds programs
+    | searchDepth cfg == 0 = case ps of
         [] -> Nothing
         (x : _) -> Just x
     | otherwise =
@@ -478,8 +568,12 @@ beamSearch df cfg outputs programs
             df
             (cfg{searchDepth = searchDepth cfg - 1})
             outputs
-            (generatePrograms (map col names) ps)
+            constants
+            conditions
+            (generatePrograms conditions vars constants ps)
   where
+    vars = map col names
+    conditions = generateConditions outputs conds (vars ++ constants ++ ps) df
     ps = pickTopN df outputs cfg $ deduplicate df programs
     names = (map fst . L.sortBy (compare `on` snd) . M.toList . columnIndices) df
 
@@ -541,6 +635,45 @@ pickTopN df (TColumn col) cfg ps =
             )
             ordered
 
+pickTopNBool ::
+    DataFrame ->
+    TypedColumn Double ->
+    [(Expr Bool, TypedColumn Bool)] ->
+    [Expr Bool]
+pickTopNBool _ _ [] = []
+pickTopNBool df (TColumn col) ps =
+    let
+        l = case toVector @Double @VU.Vector col of
+            Left e -> throw e
+            Right v -> v
+        ordered =
+            Prelude.take
+                100
+                ( map fst $
+                    L.sortBy
+                        ( \(_, c2) (_, c1) ->
+                            if maybe False isInfinite c1
+                                || maybe False isInfinite c2
+                                || maybe False isNaN c1
+                                || maybe False isNaN c2
+                                then LT
+                                else compare c1 c2
+                        )
+                        ( map
+                            (\(e, res) -> (e, getLossFunction MutualInformation l (asDoubleVector res)))
+                            ps
+                        )
+                )
+        asDoubleVector c =
+            let
+                (TColumn col') = c
+             in
+                case toVector @Bool @VU.Vector col' of
+                    Left e -> throw e
+                    Right v -> VU.map (fromIntegral @Int @Double . fromEnum) v
+     in
+        ordered
+
 satisfiesExamples :: DataFrame -> TypedColumn Double -> Expr Double -> Bool
 satisfiesExamples df col expr =
     let
@@ -548,7 +681,7 @@ satisfiesExamples df col expr =
             Left e -> throw e
             Right v -> v
      in
-        result Prelude.== col
+        result == col
 
 -- See Section 2.4 of the Haskell Report https://www.haskell.org/definition/haskell2010.pdf
 isReservedId :: T.Text -> Bool
