@@ -70,16 +70,21 @@ instance Show DataFrame where
     show :: DataFrame -> String
     show d =
         let
+            rows = 20
+            (r, c) = dataframeDimensions d
             d' =
                 d
-                    { columns = V.map (takeColumn 10) (columns d)
-                    , dataframeDimensions = (10, snd (dataframeDimensions d))
+                    { columns = V.map (takeColumn rows) (columns d)
+                    , dataframeDimensions = (min rows r, c)
                     }
+            truncationInfo =
+                "\n"
+                    ++ "Showing "
+                    ++ show (min rows r)
+                    ++ " rows out of "
+                    ++ show r
          in
-            T.unpack (asText d' False)
-                ++ "\n"
-                ++ "Showing 10 rows out of "
-                ++ show (fst (dataframeDimensions d))
+            T.unpack (asText d' False) ++ (if r > rows then truncationInfo else "")
 
 -- | For showing the dataframe as markdown in notebooks.
 toMarkdownTable :: DataFrame -> T.Text
