@@ -274,18 +274,20 @@ interpret df expression@(AggVector expr op (f :: v b -> c)) = case interpret @b
                             , errorColumnName = Nothing
                             }
                         )
-        (UnboxedColumn col) -> case testEquality (typeRep @(v b)) (typeOf col) of
+        (UnboxedColumn (col :: VU.Vector d)) -> case testEquality (typeRep @(v b)) (typeOf col) of
             Just Refl -> interpret @c df (Lit (f col))
-            Nothing ->
-                Left $
-                    TypeMismatchException
-                        ( MkTypeErrorContext
-                            { userType = Right (typeRep @(v b))
-                            , expectedType = Right (typeOf col)
-                            , callingFunctionName = Just "interpret"
-                            , errorColumnName = Nothing
-                            }
-                        )
+            Nothing -> case testEquality (typeRep @b) (typeRep @d) of
+                Just Refl -> interpret @c df (Lit (f (V.convert col)))
+                Nothing ->
+                    Left $
+                        TypeMismatchException
+                            ( MkTypeErrorContext
+                                { userType = Right (typeRep @(v b))
+                                , expectedType = Right (typeOf col)
+                                , callingFunctionName = Just "interpret"
+                                , errorColumnName = Nothing
+                                }
+                            )
 interpret df expression@(AggReduce expr op (f :: forall a. (Columnable a) => a -> a -> a)) = case interpret @a df expr of
     Left (TypeMismatchException context) ->
         Left $
@@ -862,22 +864,50 @@ interpretAggregation gdf@(Grouped df names indices os) expression@(AggFold expr
 
 instance (Num a, Columnable a) => Num (Expr a) where
     (+) :: Expr a -> Expr a -> Expr a
-    (+) = BinaryOp "add" (+)
+    (+) (Lit x) (Lit y) = Lit (x + y)
+    (+) (Lit x) expr = UnaryOp ("add " <> (T.pack . show) (Lit x)) (+ x) expr
+    (+) expr (Lit x) = UnaryOp ("add " <> (T.pack . show) (Lit x)) (+ x) expr
+    (+) e1 e2
+        | e1 == e2 = UnaryOp ("mult " <> (T.pack . show) (Lit @a 2)) (* 2) e1
+        | otherwise = BinaryOp "add" (+) e1 e2
+
+    (-) :: Expr a -> Expr a -> Expr a
+    (-) (Lit x) (Lit y) = Lit (x - y)
+    (-) e1 e2 = BinaryOp "sub" (-) e1 e2
 
     (*) :: Expr a -> Expr a -> Expr a
-    (*) = BinaryOp "mult" (*)
+    (*) (Lit 0) _ = Lit 0
+    (*) _ (Lit 0) = Lit 0
+    (*) (Lit 1) e = e
+    (*) e (Lit 1) = e
+    (*) (Lit x) (Lit y) = Lit (x * y)
+    (*) (Lit x) expr = UnaryOp ("mult " <> (T.pack . show) (Lit x)) (* x) expr
+    (*) expr (Lit x) = UnaryOp ("mult " <> (T.pack . show) (Lit x)) (* x) expr
+    (*) e1 e2
+        | e1 == e2 = UnaryOp "pow 2" (^ 2) e1
+        | otherwise = BinaryOp "mult" (*) e1 e2
 
     fromInteger :: Integer -> Expr a
     fromInteger = Lit . fromInteger
 
     negate :: Expr a -> Expr a
-    negate = UnaryOp "negate" negate
+    negate (Lit n) = Lit (negate n)
+    negate expr = case expr of
+        (UnaryOp "negate" negate e) -> case testEquality (typeOf e) (typeOf expr) of
+            Nothing -> expr -- This case is impossible.
+            Just Refl -> e
+        _ -> UnaryOp "negate" negate expr
 
     abs :: (Num a) => Expr a -> Expr a
-    abs = UnaryOp "abs" abs
+    abs expr = case expr of
+        (UnaryOp "abs" abs e) -> case testEquality (typeOf e) (typeOf expr) of
+            Nothing -> expr -- This case is impossible.
+            Just Refl -> e
+        _ -> UnaryOp "abs" abs expr
 
     signum :: (Num a) => Expr a -> Expr a
-    signum = UnaryOp "signum" signum
+    signum (Lit n) = Lit (signum n)
+    signum expr = UnaryOp "signum" signum expr
 
 add :: (Num a, Columnable a) => Expr a -> Expr a -> Expr a
 add = (+)
@@ -942,9 +972,122 @@ instance (Show a) => Show (Expr a) where
     show (AggReduce expr op _) = "(" ++ T.unpack op ++ " " ++ show expr ++ ")"
     show (AggFold expr op _ _) = "(" ++ T.unpack op ++ " " ++ show expr ++ ")"
 
-instance (Eq a, Show a) => Eq (Expr a) where
-    (==) :: (Eq a, Show a) => Expr a -> Expr a -> Bool
-    (==) l r = show l == show r
+normalize :: (Eq a, Ord a, Show a, Typeable a) => Expr a -> Expr a
+normalize expr = case expr of
+    Col name -> Col name
+    Lit val -> Lit val
+    If cond th el -> If (normalize cond) (normalize th) (normalize el)
+    UnaryOp name f e -> UnaryOp name f (normalize e)
+    BinaryOp name f e1 e2
+        | isCommutative name ->
+            let n1 = normalize e1
+                n2 = normalize e2
+             in case testEquality (typeOf n1) (typeOf n2) of
+                    Nothing -> expr
+                    Just Refl ->
+                        if compareExpr n1 n2 == GT
+                            then BinaryOp name f n2 n1 -- Swap to canonical order
+                            else BinaryOp name f n1 n2
+        | otherwise -> BinaryOp name f (normalize e1) (normalize e2)
+    AggVector e name f -> AggVector (normalize e) name f
+    AggReduce e name f -> AggReduce (normalize e) name f
+    AggNumericVector e name f -> AggNumericVector (normalize e) name f
+    AggFold e name init f -> AggFold (normalize e) name init f
+
+isCommutative :: T.Text -> Bool
+isCommutative name =
+    name
+        `elem` [ "add"
+               , "mult"
+               , "min"
+               , "max"
+               , "eq"
+               , "and"
+               , "or"
+               ]
+
+-- Compare expressions for ordering (used in normalization)
+compareExpr :: Expr a -> Expr a -> Ordering
+compareExpr e1 e2 = compare (exprKey e1) (exprKey e2)
+  where
+    exprKey :: Expr a -> String
+    exprKey (Col name) = "0:" ++ T.unpack name
+    exprKey (Lit val) = "1:" ++ show val
+    exprKey (If c t e) = "2:" ++ exprKey c ++ exprKey t ++ exprKey e
+    exprKey (UnaryOp name _ e) = "3:" ++ T.unpack name ++ exprKey e
+    exprKey (BinaryOp name _ e1 e2) = "4:" ++ T.unpack name ++ exprKey e1 ++ exprKey e2
+    exprKey (AggVector e name _) = "5:" ++ T.unpack name ++ exprKey e
+    exprKey (AggReduce e name _) = "6:" ++ T.unpack name ++ exprKey e
+    exprKey (AggNumericVector e name _) = "7:" ++ T.unpack name ++ exprKey e
+    exprKey (AggFold e name _ _) = "8:" ++ T.unpack name ++ exprKey e
+
+instance (Eq a, Columnable a) => Eq (Expr a) where
+    (==) l r = eqNormalized (normalize l) (normalize r)
+      where
+        exprEq :: (Columnable b, Columnable c) => Expr b -> Expr c -> Bool
+        exprEq e1 e2 = case testEquality (typeOf e1) (typeOf e2) of
+            Just Refl -> e1 == e2
+            Nothing -> False
+        eqNormalized :: Expr a -> Expr a -> Bool
+        eqNormalized (Col n1) (Col n2) = n1 == n2
+        eqNormalized (Lit v1) (Lit v2) = v1 == v2
+        eqNormalized (If c1 t1 e1) (If c2 t2 e2) =
+            c1 == c2 && t1 `exprEq` t2 && e1 `exprEq` e2
+        eqNormalized (UnaryOp n1 _ e1) (UnaryOp n2 _ e2) =
+            n1 == n2 && e1 `exprEq` e2
+        eqNormalized (BinaryOp n1 _ e1a e1b) (BinaryOp n2 _ e2a e2b) =
+            n1 == n2 && e1a `exprEq` e2a && e1b `exprEq` e2b
+        eqNormalized (AggVector e1 n1 _) (AggVector e2 n2 _) =
+            n1 == n2 && e1 `exprEq` e2
+        eqNormalized (AggReduce e1 n1 _) (AggReduce e2 n2 _) =
+            n1 == n2 && e1 `exprEq` e2
+        eqNormalized (AggNumericVector e1 n1 _) (AggNumericVector e2 n2 _) =
+            n1 == n2 && e1 `exprEq` e2
+        eqNormalized (AggFold e1 n1 i1 _) (AggFold e2 n2 i2 _) =
+            n1 == n2 && e1 `exprEq` e2 && i1 == i2
+        eqNormalized _ _ = False
+
+instance (Ord a, Columnable a) => Ord (Expr a) where
+    compare :: Expr a -> Expr a -> Ordering
+    compare e1 e2 = case (e1, e2) of
+        (Col n1, Col n2) -> compare n1 n2
+        (Lit v1, Lit v2) -> compare v1 v2
+        (If c1 t1 e1', If c2 t2 e2') ->
+            compare c1 c2 <> exprComp t1 t2 <> exprComp e1' e2'
+        (UnaryOp n1 _ e1', UnaryOp n2 _ e2') ->
+            compare n1 n2 <> exprComp e1' e2'
+        (BinaryOp n1 _ a1 b1, BinaryOp n2 _ a2 b2) ->
+            compare n1 n2 <> exprComp a1 a2 <> exprComp b1 b2
+        (AggVector e1' n1 _, AggVector e2' n2 _) ->
+            compare n1 n2 <> exprComp e1' e2'
+        (AggReduce e1' n1 _, AggReduce e2' n2 _) ->
+            compare n1 n2 <> exprComp e1' e2'
+        (AggNumericVector e1' n1 _, AggNumericVector e2' n2 _) ->
+            compare n1 n2 <> exprComp e1' e2'
+        (AggFold e1' n1 i1 _, AggFold e2' n2 i2 _) ->
+            compare n1 n2 <> exprComp e1' e2' <> compare i1 i2
+        -- Different constructors - compare by priority
+        (Col _, _) -> LT
+        (_, Col _) -> GT
+        (Lit _, _) -> LT
+        (_, Lit _) -> GT
+        (UnaryOp{}, _) -> LT
+        (_, UnaryOp{}) -> GT
+        (BinaryOp{}, _) -> LT
+        (_, BinaryOp{}) -> GT
+        (If{}, _) -> LT
+        (_, If{}) -> GT
+        (AggVector{}, _) -> LT
+        (_, AggVector{}) -> GT
+        (AggReduce{}, _) -> LT
+        (_, AggReduce{}) -> GT
+        (AggNumericVector{}, _) -> LT
+        (_, AggNumericVector{}) -> GT
+
+exprComp :: (Columnable b, Columnable c) => Expr b -> Expr c -> Ordering
+exprComp e1 e2 = case testEquality (typeOf e1) (typeOf e2) of
+    Just Refl -> e1 `compare` e2
+    Nothing -> LT
 
 replaceExpr ::
     forall a b c.
