@@ -11,7 +11,6 @@ module DataFrame.Operations.Core where
 import qualified Data.List as L
 import qualified Data.Map as M
 import qualified Data.Map.Strict as MS
-import qualified Data.Set as S
 import qualified Data.Text as T
 import qualified Data.Vector as V
 import qualified Data.Vector.Generic as VG
@@ -75,21 +74,21 @@ ghci> import qualified Data.Vector as V
 
 ghci> D.insertVector "numbers" (V.fromList [1..10]) D.empty
 
----------------
-index | numbers
-------|--------
- Int  |   Int
-------|--------
-0     | 1
-1     | 2
-2     | 3
-3     | 4
-4     | 5
-5     | 6
-6     | 7
-7     | 8
-8     | 9
-9     | 10
+--------
+ numbers
+--------
+   Int
+--------
+ 1
+ 2
+ 3
+ 4
+ 5
+ 6
+ 7
+ 8
+ 9
+ 10
 
 @
 -}
@@ -151,21 +150,21 @@ insertUnboxedVector name xs = insertColumn name (UnboxedColumn xs)
 @
 ghci> D.insertColumn "numbers" (D.fromList [1..10]) D.empty
 
----------------
-index | numbers
-------|--------
- Int  |   Int
-------|--------
-0     | 1
-1     | 2
-2     | 3
-3     | 4
-4     | 5
-5     | 6
-6     | 7
-7     | 8
-8     | 9
-9     | 10
+--------
+ numbers
+--------
+   Int
+--------
+ 1
+ 2
+ 3
+ 4
+ 5
+ 6
+ 7
+ 8
+ 9
+ 10
 
 @
 -}
@@ -204,21 +203,21 @@ ghci> df = insertVector "numbers" (V.fromList [1..10]) D.empty
 
 ghci> D.cloneColumn "numbers" "others" df
 
-------------------------
-index | numbers | others
-------|---------|-------
- Int  |   Int   |  Int
-------|---------|-------
-0     | 1       | 1
-1     | 2       | 2
-2     | 3       | 3
-3     | 4       | 4
-4     | 5       | 5
-5     | 6       | 6
-6     | 7       | 7
-7     | 8       | 8
-8     | 9       | 9
-9     | 10      | 10
+-----------------
+ numbers | others
+---------|-------
+   Int   |  Int
+---------|-------
+ 1       | 1
+ 2       | 2
+ 3       | 3
+ 4       | 4
+ 5       | 5
+ 6       | 6
+ 7       | 7
+ 8       | 8
+ 9       | 9
+ 10      | 10
 
 @
 -}
@@ -241,21 +240,21 @@ ghci> df = insertVector "numbers" (V.fromList [1..10]) D.empty
 
 ghci> D.rename "numbers" "others" df
 
---------------
-index | others
-------|-------
- Int  |  Int
-------|-------
-0     | 1
-1     | 2
-2     | 3
-3     | 4
-4     | 5
-5     | 6
-6     | 7
-7     | 8
-8     | 9
-9     | 10
+-------
+ others
+-------
+  Int
+-------
+ 1
+ 2
+ 3
+ 4
+ 5
+ 6
+ 7
+ 8
+ 9
+ 10
 
 @
 -}
@@ -272,39 +271,39 @@ ghci> df = D.insertVector "others" (V.fromList [11..20]) (D.insertVector "number
 
 ghci> df
 
-------------------------
-index | numbers | others
-------|---------|-------
- Int  |   Int   |  Int
-------|---------|-------
-0     | 1       | 11
-1     | 2       | 12
-2     | 3       | 13
-3     | 4       | 14
-4     | 5       | 15
-5     | 6       | 16
-6     | 7       | 17
-7     | 8       | 18
-8     | 9       | 19
-9     | 10      | 20
+-----------------
+ numbers | others
+---------|-------
+   Int   |  Int
+---------|-------
+ 1       | 11
+ 2       | 12
+ 3       | 13
+ 4       | 14
+ 5       | 15
+ 6       | 16
+ 7       | 17
+ 8       | 18
+ 9       | 19
+ 10      | 20
 
 ghci> D.renameMany [("numbers", "first_10"), ("others", "next_10")] df
 
---------------------------
-index | first_10 | next_10
-------|----------|--------
- Int  |   Int    |   Int
-------|----------|--------
-0     | 1        | 11
-1     | 2        | 12
-2     | 3        | 13
-3     | 4        | 14
-4     | 5        | 15
-5     | 6        | 16
-6     | 7        | 17
-7     | 8        | 18
-8     | 9        | 19
-9     | 10       | 20
+-------------------
+ first_10 | next_10
+----------|--------
+   Int    |   Int
+----------|--------
+ 1        | 11
+ 2        | 12
+ 3        | 13
+ 4        | 14
+ 5        | 15
+ 6        | 16
+ 7        | 17
+ 8        | 18
+ 9        | 19
+ 10       | 20
 
 @
 -}
@@ -325,8 +324,6 @@ data ColumnInfo = ColumnInfo
     { nameOfColumn :: !T.Text
     , nonNullValues :: !Int
     , nullValues :: !Int
-    , partiallyParsedValues :: !Int
-    , uniqueValues :: !Int
     , typeOfColumn :: !T.Text
     }
 
@@ -340,13 +337,13 @@ ghci> df = D.insertVector "others" (V.fromList [11..20]) (D.insertVector "number
 
 ghci> D.describeColumns df
 
------------------------------------------------------------------------------------------------------
-index | Column Name | # Non-null Values | # Null Values | # Partially parsed | # Unique Values | Type
-------|-------------|-------------------|---------------|--------------------|-----------------|-----
- Int  |    Text     |        Int        |      Int      |        Int         |       Int       | Text
-------|-------------|-------------------|---------------|--------------------|-----------------|-----
-0     | others      | 10                | 0             | 0                  | 10              | Int
-1     | numbers     | 10                | 0             | 0                  | 10              | Int
+--------------------------------------------------------
+ Column Name | # Non-null Values | # Null Values | Type
+-------------|-------------------|---------------|-----
+    Text     |        Int        |      Int      | Text
+-------------|-------------------|---------------|-----
+ others      | 10                | 0             | Int
+ numbers     | 10                | 0             | Int
 
 @
 -}
@@ -356,8 +353,6 @@ describeColumns df =
         & insertColumn "Column Name" (fromList (map nameOfColumn infos))
         & insertColumn "# Non-null Values" (fromList (map nonNullValues infos))
         & insertColumn "# Null Values" (fromList (map nullValues infos))
-        & insertColumn "# Partially parsed" (fromList (map partiallyParsedValues infos))
-        & insertColumn "# Unique Values" (fromList (map uniqueValues infos))
         & insertColumn "Type" (fromList (map typeOfColumn infos))
   where
     infos =
@@ -369,9 +364,7 @@ describeColumns df =
         let
             cname = columnName i
             countNulls = nulls col
-            countPartial = partiallyParsed col
             columnType = T.pack $ show $ typeRep @a
-            unique = S.size $ VG.foldr S.insert S.empty c
          in
             if isNothing cname
                 then acc
@@ -380,16 +373,12 @@ describeColumns df =
                         (fromMaybe "" cname)
                         (columnLength col - countNulls)
                         countNulls
-                        countPartial
-                        unique
                         columnType
                         : acc
     go acc i col@(BoxedColumn (c :: V.Vector a)) =
         let
             cname = columnName i
-            countPartial = partiallyParsed col
             columnType = T.pack $ show $ typeRep @a
-            unique = S.size $ VG.foldr S.insert S.empty c
          in
             if isNothing cname
                 then acc
@@ -398,22 +387,19 @@ describeColumns df =
                         (fromMaybe "" cname)
                         (columnLength col)
                         0
-                        countPartial
-                        unique
                         columnType
                         : acc
     go acc i col@(UnboxedColumn c) =
         let
             cname = columnName i
             columnType = T.pack $ columnTypeString col
-            unique = S.size $ VG.foldr S.insert S.empty c
          in
             -- Unboxed columns cannot have nulls since Maybe
             -- is not an instance of Unbox a
             if isNothing cname
                 then acc
                 else
-                    ColumnInfo (fromMaybe "" cname) (columnLength col) 0 0 unique columnType : acc
+                    ColumnInfo (fromMaybe "" cname) (columnLength col) 0 columnType : acc
 
 nulls :: Column -> Int
 nulls (OptionalColumn xs) = VG.length $ VG.filter isNothing xs
@@ -445,21 +431,21 @@ ghci> df = D.fromNamedColumns [("numbers", D.fromList [1..10]), ("others", D.fro
 
 ghci> df
 
-------------------------
-index | numbers | others
-------|---------|-------
- Int  |   Int   |  Int
-------|---------|-------
-0     | 1       | 11
-1     | 2       | 12
-2     | 3       | 13
-3     | 4       | 14
-4     | 5       | 15
-5     | 6       | 16
-6     | 7       | 17
-7     | 8       | 18
-8     | 9       | 19
-9     | 10      | 20
+-----------------
+ numbers | others
+---------|-------
+   Int   |  Int
+---------|-------
+ 1       | 11
+ 2       | 12
+ 3       | 13
+ 4       | 14
+ 5       | 15
+ 6       | 16
+ 7       | 17
+ 8       | 18
+ 9       | 19
+ 10      | 20
 
 @
 -}
@@ -477,20 +463,20 @@ ghci> df = D.fromUnnamedColumns [D.fromList [1..10], D.fromList [11..20]]
 ghci> df
 
 -----------------
-index |  0  |  1
-------|-----|----
- Int  | Int | Int
-------|-----|----
-0     | 1   | 11
-1     | 2   | 12
-2     | 3   | 13
-3     | 4   | 14
-4     | 5   | 15
-5     | 6   | 16
-6     | 7   | 17
-7     | 8   | 18
-8     | 9   | 19
-9     | 10  | 20
+  0  |  1
+-----|----
+ Int | Int
+-----|----
+ 1   | 11
+ 2   | 12
+ 3   | 13
+ 4   | 14
+ 5   | 15
+ 6   | 16
+ 7   | 17
+ 8   | 18
+ 9   | 19
+ 10  | 20
 
 @
 -}
@@ -505,14 +491,14 @@ ghci> df = D.fromRows ["A", "B"] [[D.toAny 1, D.toAny 11], [D.toAny 2, D.toAny 1
 
 ghci> df
 
------------------
-index |  A  |  B
-------|-----|----
- Int  | Int | Int
-------|-----|----
-0     | 1   | 11
-1     | 2   | 12
-2     | 3   | 13
+----------
+  A  |  B
+-----|----
+ Int | Int
+-----|----
+ 1   | 11
+ 2   | 12
+ 3   | 13
 
 @
 -}
@@ -597,21 +583,21 @@ This makes it easier to chain operations.
 @
 ghci> D.fold (const id) [1..5] df
 
------------------
-index |  0  |  1
-------|-----|----
- Int  | Int | Int
-------|-----|----
-0     | 1   | 11
-1     | 2   | 12
-2     | 3   | 13
-3     | 4   | 14
-4     | 5   | 15
-5     | 6   | 16
-6     | 7   | 17
-7     | 8   | 18
-8     | 9   | 19
-9     | 10  | 20
+----------
+  0  |  1
+-----|----
+ Int | Int
+-----|----
+ 1   | 11
+ 2   | 12
+ 3   | 13
+ 4   | 14
+ 5   | 15
+ 6   | 16
+ 7   | 17
+ 8   | 18
+ 9   | 19
+ 10  | 20
 
 @
 -}
