@@ -41,13 +41,13 @@ ghci> df <- D.readCsv ".\/data\/housing.csv"
 
 ghci> D.frequencies "ocean_proximity" df
 
-----------------------------------------------------------------------------
-index |   Statistic    | <1H OCEAN | INLAND | ISLAND | NEAR BAY | NEAR OCEAN
-------|----------------|-----------|--------|--------|----------|-----------
- Int  |      Text      |    Any    |  Any   |  Any   |   Any    |    Any
-------|----------------|-----------|--------|--------|----------|-----------
-0     | Count          | 9136      | 6551   | 5      | 2290     | 2658
-1     | Percentage (%) | 44.26%    | 31.74% | 0.02%  | 11.09%   | 12.88%
+---------------------------------------------------------------------
+   Statistic    | <1H OCEAN | INLAND | ISLAND | NEAR BAY | NEAR OCEAN
+----------------|-----------|--------|--------|----------|-----------
+      Text      |    Any    |  Any   |  Any   |   Any    |    Any
+----------------|-----------|--------|--------|----------|-----------
+ Count          | 9136      | 6551   | 5      | 2290     | 2658
+ Percentage (%) | 44.26%    | 31.74% | 0.02%  | 11.09%   | 12.88%
 @
 -}
 frequencies :: T.Text -> DataFrame -> DataFrame
