@@ -5,6 +5,7 @@ module Main where
 
 import qualified Data.Text as T
 import qualified Data.Vector as V
+import qualified Data.Vector.Unboxed as VU
 import qualified DataFrame as D
 import qualified DataFrame.Internal.Column as DI
 import qualified DataFrame.Operations.Typing as D
@@ -22,6 +23,7 @@ import qualified Operations.Filter
 import qualified Operations.GroupBy
 import qualified Operations.InsertColumn
 import qualified Operations.Merge
+import qualified Operations.ReadCsv
 import qualified Operations.Sort
 import qualified Operations.Statistics
 import qualified Operations.Take
@@ -65,6 +67,74 @@ parseDate =
      in
         TestCase (assertEqual "Correctly parses gregorian date" expected actual)
 
+parseInt :: Test
+parseInt =
+    let
+        expected =
+            DI.UnboxedColumn
+                ( VU.fromList
+                    [1 :: Int .. 5]
+                )
+        actual =
+            D.parseDefault
+                10
+                True
+                "%Y-%m-%d"
+                (DI.fromVector (V.fromList ["1" :: T.Text, "2", "3", "4", "5"]))
+     in
+        TestCase (assertEqual "Correctly parses integers" expected actual)
+
+parseMaybeInt :: Test
+parseMaybeInt =
+    let
+        expected =
+            DI.OptionalColumn
+                ( V.fromList
+                    [Just (1 :: Int), Nothing, Just 3, Just 4, Just 5]
+                )
+        actual =
+            D.parseDefault
+                10
+                True
+                "%Y-%m-%d"
+                (DI.fromVector (V.fromList ["1" :: T.Text, "N/A", "3", "4", "5"]))
+     in
+        TestCase (assertEqual "Correctly parses optional integers" expected actual)
+
+parseDouble :: Test
+parseDouble =
+    let
+        expected =
+            DI.UnboxedColumn
+                ( VU.fromList
+                    [1 :: Double, 2, 3, 4, 5]
+                )
+        actual =
+            D.parseDefault
+                10
+                True
+                "%Y-%m-%d"
+                (DI.fromVector (V.fromList ["1" :: T.Text, "2.0", "3", "4", "5"]))
+     in
+        TestCase (assertEqual "Correctly parses doubles" expected actual)
+
+parseMaybeDouble :: Test
+parseMaybeDouble =
+    let
+        expected =
+            DI.OptionalColumn
+                ( V.fromList
+                    [Just 1 :: Maybe Double, Nothing, Just 3, Just 4, Just 5]
+                )
+        actual =
+            D.parseDefault
+                10
+                True
+                "%Y-%m-%d"
+                (DI.fromVector (V.fromList ["1" :: T.Text, "N/A", "3.0", "4", "5"]))
+     in
+        TestCase (assertEqual "Correctly parses optional doubles" expected actual)
+
 incompleteDataParseEither :: Test
 incompleteDataParseEither =
     let
@@ -89,7 +159,7 @@ incompleteDataParseMaybe :: Test
 incompleteDataParseMaybe =
     let
         expected =
-            DI.BoxedColumn
+            DI.OptionalColumn
                 ( V.fromList
                     [Just $ fromGregorian 2020 02 14, Nothing, Just $ fromGregorian 2022 02 14]
                 )
@@ -108,6 +178,10 @@ parseTests =
     [ TestLabel "parseDate" parseDate
     , TestLabel "incompleteDataParseMaybe" incompleteDataParseMaybe
     , TestLabel "incompleteDataParseEither" incompleteDataParseEither
+    , TestLabel "parseInt" parseInt
+    , TestLabel "parseMaybeInt" parseMaybeInt
+    , TestLabel "parseDouble" parseDouble
+    , TestLabel "parseMaybeDouble" parseMaybeDouble
     ]
 
 tests :: Test
@@ -122,6 +196,7 @@ tests =
             ++ Operations.GroupBy.tests
             ++ Operations.InsertColumn.tests
             ++ Operations.Merge.tests
+            ++ Operations.ReadCsv.tests
             ++ Operations.Sort.tests
             ++ Operations.Statistics.tests
             ++ Operations.Take.tests
